==> file_extractor-2.0.0-rc2.tar.gz: file_extractor/src/Extractor/ExtractorPluginBase.php <==
<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Extractor;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StreamWrapper\StreamWrapperInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\file\FileInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface;

/**
 * Base class for FileExtractorExtractor plugins.
 *
 * @ingroup plugin_api
 */
abstract class ExtractorPluginBase extends PluginBase implements ExtractorPluginInterface, ContainerFactoryPluginInterface {

  /**
   * Name of the file used for testing.
   */
  const TEST_FILENAME = 'file_extractor_test_extraction.pdf';

  /**
   * URI of the file used for testing.
   */
  const TEST_FILE_URI = 'temporary://' . self::TEST_FILENAME;

  /**
   * Stream wrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Mime type guesser service.
   *
   * @var \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * File system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    array $plugin_definition,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    MimeTypeGuesserInterface $mime_type_guesser,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
    FileSystemInterface $file_system
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->mimeTypeGuesser = $mime_type_guesser;
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('stream_wrapper_manager'),
      $container->get('file.mime_type.guesser'),
      $container->get('logger.channel.file_extractor'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  /**
   * Form validation handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Do not trigger the test extraction if there is already an error.
    if ($form_state->getErrors()) {
      return;
    }

    $this->setConfiguration($form_state->getValues());

    // Test the extraction.
    $file = $this->getTestFile();
    $extracted_data = $this->extract($file);
    $this->fileSystem->delete(self::TEST_FILE_URI);

    if (empty($extracted_data)) {
      $this->messenger->addWarning($this->t('The extraction does not seem to work with this configuration! The attempt to extract a test PDF file returned an empty value.'));
    }
    else {
      $this->messenger->addStatus($this->t('The extraction is working with this configuration! The attempt to extract a test PDF file returned: @extracted_data.', [
        '@extracted_data' => $extracted_data,
      ]));
    }
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * Helper method to get the real path from an uri.
   *
   * @param string $uri
   *   The URI of the file, e.g. public://directory/file.jpg.
   *
   * @return string
   *   The real path to the file if it is a local file. An URL otherwise.
   */
  protected function getRealpath(string $uri) : string {
    $wrapper = $this->streamWrapperManager->getViaUri($uri);
    $scheme = $this->streamWrapperManager->getScheme($uri);
    $local_wrappers = $this->streamWrapperManager->getWrappers(StreamWrapperInterface::LOCAL);
    if (in_array($scheme, array_keys($local_wrappers))) {
      return $wrapper->realpath();
    }
    else {
      return $wrapper->getExternalUrl();
    }
  }

  /**
   * Helper method to get the PDF MIME types.
   *
   * @return array
   *   An array of the PDF MIME types.
   */
  protected function getPdfMimeTypes() : array {
    $pdf_mime_types = [];
    $pdf_mime_types[] = $this->mimeTypeGuesser->guess('dummy.pdf');
    return $pdf_mime_types;
  }

  /**
   * Check if the binary path is a path or only the binary name.
   *
   * @param string $binary_path
   *   The binary path.
   *
   * @return bool
   *   TRUE if only a binary name. FALSE otherwise.
   */
  protected function isBinaryName(string $binary_path) : bool {
    if ((strpos($binary_path, '/') === FALSE) && (strpos($binary_path, '\\') === FALSE)) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Helper function to check that a binary and a file still exists.
   *
   * @param string $binary_path
   *   The binary path.
   * @param string $file_path
   *   The path to the file to extract.
   * @param array $log_variables
   *   Log variables.
   *
   * @return bool
   *   TRUE if binary and file exist. FALSE otherwise.
   */
  protected function checkBinaryAndFile(string $binary_path, string $file_path, array &$log_variables) : bool {
    $log_variables['@binary_path'] = $binary_path;
    $log_variables['@file_path'] = $file_path;

    // Test that the binary is still available.
    if (!$this->isBinaryName($binary_path) && !file_exists($binary_path)) {
      $this->logger->error('Extractor binary was not found at @binary_path.', $log_variables);
      return FALSE;
    }

    // Test that the file is still available.
    if (!file_exists($file_path)) {
      $this->logger->error('File was not found at @file_path.', $log_variables);
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Helper function to run shell command.
   *
   * @param string $command
   *   The command. This will be passed to escapeshellcmd().
   * @param array $command_arguments
   *   The command arguments. They will have to be manually passed to
   *   escapeshellarg() if needed.
   * @param string $command_prefix
   *   A prefix to the command. Will no go through escapeshellcmd().
   *   You must had a space at the end because by default there is no space
   *   between $command_prefix and $command when concatenating.
   *
   * @return string|null
   *   The shell_exec return.
   */
  protected function execExtraction(string $command, array $command_arguments = [], string $command_prefix = '') {
    // UTF-8 multibyte characters will be stripped by escapeshellargs() for
    // the default C-locale.
    // So temporarily set the locale to UTF-8 so that the filepath remains
    // valid.
    $backup_locale = setlocale(LC_CTYPE, '0');
    setlocale(LC_CTYPE, 'en_US.UTF-8');

    $cmd = $command_prefix . escapeshellcmd($command) . ' ' . implode(' ', $command_arguments);

    // Restore the locale.
    setlocale(LC_CTYPE, $backup_locale);
    // Support UTF-8 commands.
    // @see http://www.php.net/manual/en/function.shell-exec.php#85095
    shell_exec("LANG=en_US.utf-8");

    return shell_exec($cmd);
  }

  /**
   * Helper method to get a pdf test file and extract its data.
   *
   * @return \Drupal\file\FileInterface
   *   A file entity object.
   */
  protected function getTestFile() : FileInterface {
    // Copy the source file to public directory.
    $source = drupal_get_path('module', 'file_extractor');
    $source .= '/data/' . self::TEST_FILENAME;
    $this->fileSystem->copy($source, self::TEST_FILE_URI, FileSystemInterface::EXISTS_REPLACE);

    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entityTypeManager->getStorage('file')->create([
      'filename' => self::TEST_FILENAME,
      'filemime' => 'application/pdf',
      'uri' => self::TEST_FILE_URI,
    ]);
    return $file;
  }

}

==> file_extractor-2.0.1.tar.gz: file_extractor/src/Extractor/ExtractorPluginBase.php <==
<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Extractor;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StreamWrapper\StreamWrapperInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\file_extractor\Service\TestFileHelperInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface;

/**
 * Base class for FileExtractorExtractor plugins.
 *
 * @ingroup plugin_api
 */
abstract class ExtractorPluginBase extends PluginBase implements ExtractorPluginInterface, ContainerFactoryPluginInterface {

  /**
   * Stream wrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Mime type guesser service.
   *
   * @var \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Test file helper.
   *
   * @var \Drupal\file_extractor\Service\TestFileHelperInterface
   */
  protected $testFileHelper;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    array $plugin_definition,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    MimeTypeGuesserInterface $mime_type_guesser,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
    TestFileHelperInterface $test_file_helper
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->mimeTypeGuesser = $mime_type_guesser;
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->testFileHelper = $test_file_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('stream_wrapper_manager'),
      $container->get('file.mime_type.guesser'),
      $container->get('logger.channel.file_extractor'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('file_extractor.test_file_helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  /**
   * Form validation handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Do not trigger the test extraction if there is already an error.
    if ($form_state->getErrors()) {
      return;
    }

    $this->setConfiguration($form_state->getValues());

    // Test the extraction.
    $extracted_data = $this->extract($this->testFileHelper->getTestFile());
    $this->testFileHelper->deleteTestFile();

    if (empty($extracted_data)) {
      $this->messenger->addWarning($this->t('The extraction does not seem to work with this configuration! The attempt to extract a test PDF file returned an empty value.'));
    }
    else {
      $this->messenger->addStatus($this->t('The extraction is working with this configuration! The attempt to extract a test PDF file returned: @extracted_data.', [
        '@extracted_data' => $extracted_data,
      ]));
    }
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the plugin form as built
   *   by static::buildConfigurationForm().
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the complete form.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * Helper method to get the real path from an uri.
   *
   * @param string $uri
   *   The URI of the file, e.g. public://directory/file.jpg.
   *
   * @return string
   *   The real path to the file if it is a local file. An URL otherwise.
   */
  protected function getRealpath(string $uri) : string {
    $wrapper = $this->streamWrapperManager->getViaUri($uri);
    $scheme = $this->streamWrapperManager->getScheme($uri);
    $local_wrappers = $this->streamWrapperManager->getWrappers(StreamWrapperInterface::LOCAL);
    if (in_array($scheme, array_keys($local_wrappers))) {
      return $wrapper->realpath();
    }
    else {
      return $wrapper->getExternalUrl();
    }
  }

  /**
   * Helper method to get the PDF MIME types.
   *
   * @return array
   *   An array of the PDF MIME types.
   */
  protected function getPdfMimeTypes() : array {
    $pdf_mime_types = [];
    $pdf_mime_types[] = $this->mimeTypeGuesser->guess('dummy.pdf');
    return $pdf_mime_types;
  }

  /**
   * Check if the binary path is a path or only the binary name.
   *
   * @param string $binary_path
   *   The binary path.
   *
   * @return bool
   *   TRUE if only a binary name. FALSE otherwise.
   */
  protected function isBinaryName(string $binary_path) : bool {
    if ((strpos($binary_path, '/') === FALSE) && (strpos($binary_path, '\\') === FALSE)) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Helper function to check that a binary and a file still exists.
   *
   * @param string $binary_path
   *   The binary path.
   * @param string $file_path
   *   The path to the file to extract.
   * @param array $log_variables
   *   Log variables.
   *
   * @return bool
   *   TRUE if binary and file exist. FALSE otherwise.
   */
  protected function checkBinaryAndFile(string $binary_path, string $file_path, array &$log_variables) : bool {
    $log_variables['@binary_path'] = $binary_path;
    $log_variables['@file_path'] = $file_path;

    // Test that the binary is still available.
    if (!$this->isBinaryName($binary_path) && !file_exists($binary_path)) {
      $this->logger->error('Extractor binary was not found at @binary_path.', $log_variables);
      return FALSE;
    }

    // Test that the file is still available.
    if (!file_exists($file_path)) {
      $this->logger->error('File was not found at @file_path.', $log_variables);
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Helper function to run shell command.
   *
   * @param string $command
   *   The command. This will be passed to escapeshellcmd().
   * @param array $command_arguments
   *   The command arguments. They will have to be manually passed to
   *   escapeshellarg() if needed.
   * @param string $command_prefix
   *   A prefix to the command. Will no go through escapeshellcmd().
   *   You must had a space at the end because by default there is no space
   *   between $command_prefix and $command when concatenating.
   *
   * @return string|null
   *   The shell_exec return.
   */
  protected function execExtraction(string $command, array $command_arguments = [], string $command_prefix = '') {
    // UTF-8 multibyte characters will be stripped by escapeshellargs() for
    // the default C-locale.
    // So temporarily set the locale to UTF-8 so that the filepath remains
    // valid.
    $backup_locale = setlocale(LC_CTYPE, '0');
    setlocale(LC_CTYPE, 'en_US.UTF-8');

    $cmd = $command_prefix . escapeshellcmd($command) . ' ' . implode(' ', $command_arguments);

    // Restore the locale.
    setlocale(LC_CTYPE, $backup_locale);
    // Support UTF-8 commands.
    // @see http://www.php.net/manual/en/function.shell-exec.php#85095
    shell_exec("LANG=en_US.utf-8");

    return shell_exec($cmd);
  }

}
